/***************************************************************************
 *   Copyright (C) 2009 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUS_IPV4UDPLITE_H
#define CONEXUS_IPV4UDPLITE_H

#include <conexus/ipv4_udp.h>

#ifdef IPPROTO_UDPLITE

/**
 * \example ipv4_udplite_client.cpp
 * This is an example of how to use the Conexus::IPv4::UDPLite class.
 *
 * \example ipv4_udplite_server.cpp
 * This is an example of how to use the Conexus::IPv4::UDPLite class.
 */

namespace Conexus
{

  namespace IPv4
  {

    /**
     * IPv4 UDP-Lite endpoint as described in RFC 3828
     *
     * \par Summary
     * UDP-Lite is an extension of UDP that provides a checksum with optional
     * partial coverage. A packet is divided into a sensitive part (covered by
     * the checksum) and an insensitive part (not covered by the checksum).
     *
     * Checksum errors in the insensitive part will not be discarded.
     *
     * \par Coverage values
     * A coverage value of 0 indicates that the entire datagram is covered by the
     * checksum. Thus, the entire datagram is considered sensitive. This behavior
     * is compatible with and identical to UDP.
     *
     * \par
     * Coverage values from 1-7 are illegal and are rounded up to 8. Thus, the
     * UDP header is always considered sensitive and is always covered by the
     * checksum. The payload begins in the 9th octet.
     *
     * \par
     * Since the UDP-Lite protocol uses the UDP length field to convey the
     * checksum coverage value (UDP-Lite obtains the length from the IP
     * header) the maximum coverage size is 2^16-1. This also applies to
     * IPv6 jumbograms.
     *
     * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
     * @ingroup IPv4
     * @ingroup Endpoint
     */
    class UDPLite : public UDP
    {
      protected:

        UDPLite( uint32_t local_if, uint16_t localport, uint16_t remoteport );

        UDPLite( const std::string& local_if, uint16_t localport, uint16_t remoteport );

      public:

        typedef ConexusPointer<UDPLite> pointer;

        CONEXUS_ENDPOINT_GENERIC_CREATE();

        static pointer create( uint16_t localport = 0, uint16_t remoteport = 0 );

        static pointer create( uint32_t local_if, uint16_t localport, uint16_t remoteport );

        static pointer create( const std::string& local_if, uint16_t localport = 0, uint16_t remoteport = 0 );

        virtual ~UDPLite() throw();

        /** Returns the coverage of sent datagrams */
        uint16_t send_coverage();

        /** Sets the coverage of sent datagrams */
        uint16_t set_send_coverage(uint16_t coverage_octets);

        /**
         * Convenience method to sets the number of payload octets covered on sent datagrams
         *
         * The first 8 octets cover the UDP-Lite header. Therefore the payload
         * will only be covered by values of 9 or higher (or 0 in the case of
         * full datagram coverage).
         *
         * This is a shorthand that always adds 8 to the value. If you want
         * full datagram coverage use \c set_send_coverage(0) since
         * \c set_payload_send_coverage(0) is equivalent to
         * \c set_send_coverage(8) which is \b not full coverage.
         */
        uint16_t set_payload_send_coverage(uint16_t coverage_octets);

        /** Returns the coverage of received datagrams */
        uint16_t recv_coverage();

        /**
         * Sets the coverage of received datagrams. Setting this is not required
         * for to enable traffic with partial checksum coverage.
         *
         * In essence, this acts as a filter. When enabled, the kernel will drop
         * all datagrams that have coverage less than the specified value.
         */
        uint16_t set_recv_coverage(uint16_t coverage_octets);

        /**
         * Convenience method to sets the number of payload octets covered on received datagrams
         *
         * The first 8 octets cover the UDP-Lite header. Therefore the payload
         * will only be covered by values of 9 or higher (or 0 in the case of
         * full datagram coverage).
         *
         * This is a shorthand that always adds 8 to the value. If you want
         * full datagram coverage use \c set_recv_coverage(0) since
         * \c set_payload_recv_coverage(0) is equivalent to
         * \c set_recv_coverage(8) which is \b not full coverage.
         */
        uint16_t set_payload_recv_coverage(uint16_t coverage_octets);

    };

  }

}

#endif

#endif
