/***************************************************************************
 *   Copyright (C) 2001,2002,2003,2004,2005,2006,2007,2008,2009            *
 *   by Rick L. Vinyard, Jr.                                               *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "network_device.h"

#include "except.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

namespace Conexus
{
  namespace Network
  {
    
    int local_socket = -1;

    bool check_local_socket()
    {
      if ( local_socket != -1 ) return true;
      local_socket = socket(AF_UNIX, SOCK_STREAM, 0);
      return local_socket != -1;
    }

    unsigned interface_index( const std::string& name )
    {
      unsigned index = if_nametoindex( name.c_str() );
      if ( index == 0 )
        throw interface_index_exception();
      return index;
    }

    std::string interface_name( unsigned ifindex )
    {
      char index_name[IF_NAMESIZE];

      if ( index_name != if_indextoname( ifindex, index_name ) )
        throw interface_index_exception();

      return std::string( index_name );
    }

    std::vector<std::pair<unsigned,std::string> > interfaces()
    {
      struct if_nameindex *index, *i;
      std::vector<std::pair<unsigned,std::string> > ifs;

      index = if_nameindex();
      for ( i = index; i != NULL; i++ )
        ifs.push_back( std::pair<unsigned,std::string>(i->if_index, i->if_name) );
      if_freenameindex(index);

      return ifs;
    }

    std::vector<std::string> interface_names()
    {
      struct if_nameindex *index, *i;
      std::vector<std::string> ifs;

      index = if_nameindex();
      for ( i = index; i != NULL; i++ )
        ifs.push_back( std::string(i->if_name) );
      if_freenameindex(index);

      return ifs;
    }

#define MAX_REQ_SIZE 100

    std::vector< std::pair < std::string , std::string > > interface_ip_addresses( )
    {
      std::vector< std::pair<std::string, std::string> > ifaddrs;

      int sd = socket( AF_UNIX, SOCK_STREAM, 0 );
      struct ifreq* req;
      struct ifconf conf;
      char buf[ sizeof( struct ifreq ) * MAX_REQ_SIZE ];
      std::pair<std::string, std::string> interface;
      struct sockaddr_in* sa;

      conf.ifc_len = sizeof( buf );
      conf.ifc_buf = ( caddr_t ) buf;

      if ( ioctl( sd, SIOCGIFCONF, ( caddr_t ) & conf ) >= 0 )
        for ( req = ( struct ifreq* ) conf.ifc_buf; ( caddr_t ) req < conf.ifc_buf + conf.ifc_len; req++ ) {
        interface.first = std::string( req->ifr_name );
        if ( req->ifr_addr.sa_family == AF_INET ) {
          sa = ( struct sockaddr_in* ) & ( req->ifr_addr );
          interface.second = std::string( inet_ntoa( sa->sin_addr ) );
          ifaddrs.push_back( interface );
        }
        }

        close( sd );
        return ifaddrs;
    }

    Device::Device(const std::string & name):
        m_name(name),
        m_index(-1)
    {
    }

    Device::Device(int index):
        m_index(index)
    {
    }

    Device::~Device()
    {
    }

    int Device::index()
    {
      if ( m_index >= 0 ) return m_index;
      if ( m_name.empty() ) return -1;
      if ( not check_local_socket() ) return -1;
      if ( ioctl( local_socket, SIOCGIFINDEX, &m_ifreq ) < 0 ) return -1;
      return m_ifreq.ifr_ifindex;
    }

    void Device::set_index(int index)
    {
      m_index = index;
      if ( m_index >= 0 and check_local_socket() )
      {
        m_ifreq.ifr_ifindex = m_index;
        if ( ioctl( local_socket, SIOCGIFNAME, &m_ifreq ) >= 0 )
        {
          m_name = m_ifreq.ifr_name;
        }
      }
    }

    std::string Device::name()
    {
      return m_name;
    }

    void Device::set_name(const std::string & name)
    {
      m_name = name;
      strncpy(m_ifreq.ifr_name, m_name.c_str(), IFNAMSIZ);
      m_index = -1;
    }

    bool Device::is_up()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_UP;
    }

    bool Device::has_broadcast()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_BROADCAST;
    }

    bool Device::is_debug()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_DEBUG;
    }

    bool Device::is_loopback()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_LOOPBACK;
    }

    bool Device::is_point_to_point()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_POINTOPOINT;
    }

    bool Device::is_running()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_RUNNING;
    }

    bool Device::no_arp()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_NOARP;
    }

    bool Device::is_promiscuous()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_PROMISC;
    }

    bool Device::no_trailers()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_NOTRAILERS;
    }

    bool Device::receives_all_multicast()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_ALLMULTI;
    }

    bool Device::is_master()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_MASTER;
    }

    bool Device::is_slave()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_SLAVE;
    }

    bool Device::is_multicast_capable()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_MULTICAST;
    }

    bool Device::port_selectable()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_PORTSEL;
    }

    bool Device::auto_media_active()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_AUTOMEDIA;
    }

    bool Device::is_dynamic()
    {
      if ( m_name.empty() ) return false;
      if ( not check_local_socket() ) return false;
      if ( ioctl( local_socket, SIOCGIFFLAGS, &m_ifreq ) < 0 ) return false;
      return m_ifreq.ifr_flags & IFF_DYNAMIC;
    }

  }
}

