/***************************************************************************
 *   Copyright (C) 2009 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "timeval.h"

#define MICROSECONDS 1000000
#define MICROSECONDSF 1000000.0f
#define MICROSECONDSD 1000000.0
#define MICROSECONDSL 1000000.0l

namespace Conexus
{

  TimeVal::TimeVal ( int64_t seconds, int64_t microseconds ) :
      m_seconds ( seconds ),
      m_microseconds ( microseconds )
  {
  }

  TimeVal::TimeVal ( const struct timeval & tv ) :
      m_seconds ( tv.tv_sec ),
      m_microseconds ( tv.tv_usec )
  {
  }

  TimeVal::TimeVal ( const TimeVal& other ) :
      m_seconds ( other.m_seconds ),
      m_microseconds ( other.m_microseconds )
  {
  }

  TimeVal::~TimeVal()
  {
  }

  int64_t TimeVal::seconds() const
  {
    return m_seconds;
  }

  int64_t TimeVal::microseconds() const
  {
    return m_microseconds;
  }

  void TimeVal::set_seconds ( int64_t seconds )
  {
    m_seconds = seconds;
    this->cleanup();
  }

  void TimeVal::set_microseconds ( int64_t microseconds )
  {
    m_microseconds = microseconds;
    this->cleanup();
  }

  void TimeVal::set ( int64_t seconds, int64_t microseconds )
  {
    m_seconds = seconds;
    m_microseconds = microseconds;
    this->cleanup();
  }

  TimeVal & TimeVal::operator = ( const struct timeval & tv )
  {
    m_seconds = tv.tv_sec;
    m_microseconds = tv.tv_usec;
    this->cleanup();
    return *this;
  }

  void TimeVal::clear()
  {
    m_seconds = 0;
    m_microseconds = 0;
  }

  bool TimeVal::set_to_current_time()
  {
    struct timeval tv;
    if ( gettimeofday ( &tv, NULL ) == 0 )
    {
      m_seconds = tv.tv_sec;
      m_microseconds = tv.tv_usec;
      return true;
    }
    return false;
  }

  void TimeVal::add ( const TimeVal & other )
  {
    m_seconds += other.m_seconds;
    m_microseconds += other.m_microseconds;
    this->cleanup();
  }

  void TimeVal::subtract ( const TimeVal & other )
  {
    m_seconds -= other.m_seconds;
    m_microseconds -= other.m_microseconds;
    this->cleanup();
  }

  void TimeVal::add_seconds ( int64_t seconds )
  {
    m_seconds += seconds;
    this->cleanup();
  }

  void TimeVal::subtract_seconds ( int64_t seconds )
  {
    m_seconds -= seconds;
    this->cleanup();
  }

  void TimeVal::add_milliseconds ( int64_t milliseconds )
  {
    m_microseconds += milliseconds * 1000;
    this->cleanup();
  }

  void TimeVal::subtract_milliseconds ( int64_t milliseconds )
  {
    m_microseconds -= milliseconds * 1000;
    this->cleanup();
  }

  void TimeVal::add_microseconds ( int64_t microseconds )
  {
    m_microseconds += microseconds;
    this->cleanup();
  }

  void TimeVal::subtract_microsecond ( int64_t microseconds )
  {
    m_microseconds -= microseconds;
    this->cleanup();
  }

  TimeVal TimeVal::operator + ( const TimeVal & other ) const
  {
    TimeVal t ( *this );
    t += other;
    return t;
  }

  TimeVal TimeVal::operator - ( const TimeVal & other ) const
  {
    TimeVal t ( *this );
    t -= other;
    return t;
  }

  TimeVal TimeVal::operator + ( int64_t seconds ) const
  {
    TimeVal t ( *this );
    t += seconds;
    return t;
  }

  TimeVal TimeVal::operator - ( int64_t seconds ) const
  {
    TimeVal t ( *this );
    t -= seconds;
    return t;
  }

  TimeVal & TimeVal::operator += ( const TimeVal & other )
  {
    this->add(other);
    return *this;
  }

  TimeVal & TimeVal::operator -= ( const TimeVal & other )
  {
    this->subtract(other);
    return *this;
  }

  TimeVal & TimeVal::operator += ( int64_t seconds )
  {
    this->add_seconds(seconds);
    return *this;
  }

  TimeVal & TimeVal::operator -= ( int64_t seconds )
  {
    this->subtract_seconds(seconds);
    return *this;
  }

  float TimeVal::as_float() const
  {
    float d;
    d = m_seconds + m_microseconds / MICROSECONDSF;
    return d;
  }

  double TimeVal::as_double() const
  {
    double d;
    d = m_seconds + m_microseconds / MICROSECONDSD;
    return d;
  }

  long double TimeVal::as_long_double() const
  {
    long double d;
    d = m_seconds + m_microseconds / MICROSECONDSL;
    return d;
  }

  TimeVal::operator struct timeval() const
    {
      struct timeval tv;
      tv.tv_sec = m_seconds;
      tv.tv_usec = m_microseconds;
      return tv;
    }

  TimeVal::operator float() const
  {
    return this->as_float();
  }

  TimeVal::operator double() const
  {
    return this->as_double();
  }

  TimeVal::operator long double() const
  {
    return this->as_long_double();
  }

  bool TimeVal::negative() const
  {
    return ( m_seconds < 0 or m_microseconds < 0 );
  }

  void TimeVal::cleanup()
  {
    if ( m_microseconds >= MICROSECONDS or m_microseconds <= -MICROSECONDS )
    {
      m_seconds += m_microseconds / MICROSECONDS;
      m_microseconds = m_microseconds % MICROSECONDS;
    }

    if ( m_seconds > 0 and m_microseconds < 0 )
    {
      m_microseconds = MICROSECONDS - m_microseconds;
      m_seconds--;
    }

    if ( m_seconds < 0 and m_microseconds > 0 )
    {
      m_microseconds = - MICROSECONDS - m_microseconds;
      m_seconds++;
    }
  }

}

